@@ -32,8 +32,13 @@ extern "C" {
 
 #include <inttypes.h>
 #include <stddef.h>
+#include <stdint.h>
 #include <ucontext.h>
 
+#ifndef UNW_EMPTY_STRUCT
+#  define UNW_EMPTY_STRUCT uint8_t unused;
+#endif
+
 #define UNW_TARGET      sh
 #define UNW_TARGET_SH   1
 
@@ -52,6 +57,8 @@ typedef int32_t unw_sword_t;
 
 typedef long double unw_tdep_fpreg_t;
 
+#define UNW_WORD_MAX UINT32_MAX
+
 typedef enum
   {
     UNW_SH_R0,
@@ -91,6 +98,7 @@ typedef ucontext_t unw_tdep_context_t;
 typedef struct unw_tdep_save_loc
   {
     /* Additional target-dependent info on a save location.  */
+    UNW_EMPTY_STRUCT
   }
 unw_tdep_save_loc_t;
 
@@ -99,6 +107,7 @@ unw_tdep_save_loc_t;
 typedef struct
   {
     /* no sh-specific auxiliary proc-info */
+    UNW_EMPTY_STRUCT
   }
 unw_tdep_proc_info_t;
 
