@@ -32,8 +32,13 @@ extern "C" {
 
 #include <sys/types.h>
 #include <inttypes.h>
+#include <stdint.h>
 #include <ucontext.h>
 
+#ifndef UNW_EMPTY_STRUCT
+#  define UNW_EMPTY_STRUCT uint8_t unused;
+#endif
+
 #define UNW_TARGET      x86
 #define UNW_TARGET_X86  1
 
@@ -49,6 +54,8 @@ extern "C" {
 typedef uint32_t unw_word_t;
 typedef int32_t unw_sword_t;
 
+#define UNW_WORD_MAX UINT32_MAX
+
 typedef union {
   struct { uint8_t b[4]; } val32;
   struct { uint8_t b[10]; } val80;
@@ -158,6 +165,7 @@ x86_regnum_t;
 typedef struct unw_tdep_save_loc
   {
     /* Additional target-dependent info on a save location.  */
+    UNW_EMPTY_STRUCT
   }
 unw_tdep_save_loc_t;
 
@@ -169,6 +177,7 @@ typedef ucontext_t unw_tdep_context_t;
 typedef struct
   {
     /* no x86-specific auxiliary proc-info */
+    UNW_EMPTY_STRUCT
   }
 unw_tdep_proc_info_t;
 
