@@ -37,6 +37,7 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 
 #include <stdlib.h>
 #include <libunwind.h>
+#include <stdatomic.h>
 
 #include "elf32.h"
 #include "mempool.h"
@@ -51,12 +52,11 @@ unw_tdep_frame_t;
 struct unw_addr_space
 {
   struct unw_accessors acc;
-  unw_caching_policy_t caching_policy;
-#ifdef HAVE_ATOMIC_OPS_H
-  AO_t cache_generation;
-#else
-  uint32_t cache_generation;
+#ifndef UNW_REMOTE_ONLY
+  unw_iterate_phdr_func_t iterate_phdr_function;
 #endif
+  unw_caching_policy_t caching_policy;
+  _Atomic uint32_t cache_generation;
   unw_word_t dyn_generation;    /* see dyn-common.h */
   unw_word_t dyn_info_list_addr;        /* (cached) dyn_info_list_addr */
   struct dwarf_rs_cache global_cache;
@@ -260,11 +260,12 @@ dwarf_put (struct dwarf_cursor *c, dwarf_loc_t loc, unw_word_t val)
 #define tdep_find_unwind_table          dwarf_find_unwind_table
 #define tdep_uc_addr                    UNW_ARCH_OBJ(uc_addr)
 #define tdep_get_elf_image              UNW_ARCH_OBJ(get_elf_image)
+#define tdep_get_exe_image_path         UNW_ARCH_OBJ(get_exe_image_path)
 #define tdep_access_reg                 UNW_OBJ(access_reg)
 #define tdep_access_fpreg               UNW_OBJ(access_fpreg)
 #define tdep_fetch_frame(c,ip,n)        do {} while(0)
-#define tdep_cache_frame(c,rs)          do {} while(0)
-#define tdep_reuse_frame(c,rs)          do {} while(0)
+#define tdep_cache_frame(c)             0
+#define tdep_reuse_frame(c,frame)       do {} while(0)
 #define tdep_stash_frame(c,rs)          do {} while(0)
 #define tdep_trace(cur,addr,n)          (-UNW_ENOINFO)
 #define tdep_get_func_addr              UNW_OBJ(get_func_addr)
@@ -291,7 +292,7 @@ extern int tdep_fetch_proc_info_post (struct dwarf_cursor *c, unw_word_t ip,
 #define tdep_get_ip(c)                  ((c)->dwarf.ip)
 #define tdep_big_endian(as)             1
 
-extern int tdep_init_done;
+extern atomic_bool tdep_init_done;
 
 extern void tdep_init (void);
 extern int tdep_search_unwind_table (unw_addr_space_t as, unw_word_t ip,
@@ -302,6 +303,7 @@ extern void *tdep_uc_addr (ucontext_t * uc, int reg);
 extern int tdep_get_elf_image (struct elf_image *ei, pid_t pid, unw_word_t ip,
                                unsigned long *segbase, unsigned long *mapoff,
                                char *path, size_t pathlen);
+extern void tdep_get_exe_image_path (char *path);
 extern int tdep_access_reg (struct cursor *c, unw_regnum_t reg,
                             unw_word_t * valp, int write);
 extern int tdep_access_fpreg (struct cursor *c, unw_regnum_t reg,
