@@ -21,8 +21,10 @@
 # include "tdep-x86/jmpbuf.h"
 #elif defined __x86_64__
 # include "tdep-x86_64/jmpbuf.h"
-#elif defined __tilegx__
-# include "tdep-tilegx/jmpbuf.h"
+#elif defined __riscv || defined __riscv__
+# include "tdep-riscv/jmpbuf.h"
+#elif defined __loongarch64
+# include "tdep-loongarch64/jmpbuf.h"
 #else
 # error "Unsupported arch"
 #endif
