@@ -37,7 +37,7 @@ get_mem(size_t sz)
 {
   void *res;
 
-  res = mmap(NULL, sz, PROT_READ | PROT_WRITE, MAP_ANON | MAP_PRIVATE, -1, 0);
+  res = mi_mmap(NULL, sz, PROT_READ | PROT_WRITE, MAP_ANON | MAP_PRIVATE, -1, 0);
   if (res == MAP_FAILED)
     return (NULL);
   return (res);
@@ -46,7 +46,7 @@ get_mem(size_t sz)
 static void
 free_mem(void *ptr, size_t sz)
 {
-  munmap(ptr, sz);
+  mi_munmap(ptr, sz);
 }
 
 static int
@@ -56,7 +56,7 @@ get_pid_by_tid(int tid)
   size_t len, len1;
   char *buf;
   struct kinfo_proc *kv;
-  int i, pid;
+  unsigned i, pid;
 
   len = 0;
   mib[0] = CTL_KERN;
@@ -88,7 +88,7 @@ get_pid_by_tid(int tid)
   return (pid);
 }
 
-PROTECTED int
+int
 tdep_get_elf_image (struct elf_image *ei, pid_t pid, unw_word_t ip,
                     unsigned long *segbase, unsigned long *mapoff, char *path, size_t pathlen)
 {
@@ -136,10 +136,35 @@ tdep_get_elf_image (struct elf_image *ei, pid_t pid, unw_word_t ip,
      if (path)
        {
          strncpy(path, kv->kve_path, pathlen);
+         path[pathlen - 1] = '\0';
        }
-     ret = elf_map_image (ei, kv->kve_path);
+     if (ei)
+       ret = elf_map_image (ei, kv->kve_path);
+     else
+       ret = strlen (kv->kve_path) >= pathlen ? -UNW_ENOMEM : UNW_ESUCCESS;
      break;
   }
   free_mem(buf, len1);
   return (ret);
 }
+
+#ifndef UNW_REMOTE_ONLY
+
+void
+tdep_get_exe_image_path (char *path)
+{
+  int mib[4], error;
+  size_t len;
+
+  len = 0;
+  mib[0] = CTL_KERN;
+  mib[1] = KERN_PROC;
+  mib[2] = KERN_PROC_PATHNAME;
+  mib[3] = getpid();
+
+  error = sysctl(mib, 4, path, &len, NULL, 0);
+  if (error == -1)
+	  path[0] = 0;
+}
+
+#endif
