@@ -49,7 +49,7 @@ typedef struct
 } stack_frame_t;
 
 
-PROTECTED int
+int
 unw_step (unw_cursor_t * cursor)
 {
   struct cursor *c = (struct cursor *) cursor;
@@ -57,6 +57,7 @@ unw_step (unw_cursor_t * cursor)
   unw_word_t back_chain_offset, lr_save_offset;
   struct dwarf_loc back_chain_loc, lr_save_loc, sp_loc, ip_loc;
   int ret;
+  int validate = c->dwarf.as->validate;
 
   Debug (1, "(cursor=%p, ip=0x%016lx)\n", c, (unsigned long) c->dwarf.ip);
 
@@ -69,7 +70,9 @@ unw_step (unw_cursor_t * cursor)
 
   /* Try DWARF-based unwinding... */
 
+  c->dwarf.as->validate = 1;
   ret = dwarf_step (&c->dwarf);
+  c->dwarf.as->validate = validate;
 
   if (ret < 0 && ret != -UNW_ENOINFO)
     {
@@ -79,7 +82,7 @@ unw_step (unw_cursor_t * cursor)
 
   if (unlikely (ret < 0))
     {
-      if (likely (!unw_is_signal_frame (cursor)))
+      if (likely (unw_is_signal_frame (cursor) <= 0))
         {
           /* DWARF unwinding failed.  As of 09/26/2006, gcc in 64-bit mode
              produces the mandatory level of traceback record in the code, but
