@@ -1,6 +1,6 @@
 /* libunwind - a platform-independent unwind library
    Copyright (C) 2003, 2005 Hewlett-Packard Co
-	Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
+        Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
 This file is part of libunwind.
 
@@ -26,32 +26,34 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 #ifndef _UPT_internal_h
 #define _UPT_internal_h
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_PTRACE_H
+#include <sys/ptrace.h>
+#endif
+#ifdef HAVE_SYS_PROCFS_H
+#include <sys/procfs.h>
+#endif
+
 #include <errno.h>
 #include <libunwind-ptrace.h>
 #include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <sys/ptrace.h>
-
 #include "libunwind_i.h"
 
 struct UPT_info
   {
-    pid_t pid;		/* the process-id of the child we're unwinding */
-    struct elf_image ei;
-    unw_dyn_info_t di_cache;
-#if UNW_TARGET_IA64
-    unw_dyn_info_t ktab;
-#endif
+    pid_t pid;          /* the process-id of the child we're unwinding */
+    struct elf_dyn_info edi;
   };
 
-extern int _UPT_reg_offset[UNW_REG_LAST + 1];
-
-extern unw_dyn_info_t *_UPTi_find_unwind_table (struct UPT_info *ui,
-						unw_addr_space_t as,
-						char *path,
-						unw_word_t segbase,
-						unw_word_t mapoff);
+extern const int _UPT_reg_offset[UNW_REG_LAST + 1];
 
 #endif /* _UPT_internal_h */
