@@ -25,7 +25,7 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 
 #include "unwind-internal.h"
 
-PROTECTED void
+void
 _Unwind_SetIP (struct _Unwind_Context *context, unsigned long new_value)
 {
   unw_set_reg (&context->cursor, UNW_REG_IP, new_value);
