@@ -26,17 +26,18 @@ OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 
 #include "init.h"
+#include "libunwind_i.h"
 #include "unwind_i.h"
 
-PROTECTED int
-unw_init_remote (unw_cursor_t *cursor, unw_addr_space_t as, void *as_arg)
+int
+unw_init_remote (unw_cursor_t *cursor UNUSED, unw_addr_space_t as UNUSED, void *as_arg UNUSED)
 {
 #ifdef UNW_LOCAL_ONLY
   return -UNW_EINVAL;
 #else /* !UNW_LOCAL_ONLY */
   struct cursor *c = (struct cursor *) cursor;
 
-  if (!tdep_init_done)
+  if (!atomic_load(&tdep_init_done))
     tdep_init ();
 
   Debug (1, "(cursor=%p)\n", c);
@@ -44,13 +45,11 @@ unw_init_remote (unw_cursor_t *cursor, unw_addr_space_t as, void *as_arg)
   c->dwarf.as = as;
   if (as == unw_local_addr_space)
     {
-      c->dwarf.as_arg = c;
-      c->uc = as_arg;
+      c->dwarf.as_arg = dwarf_build_as_arg(as_arg, /*validate*/ 0);
     }
   else
     {
       c->dwarf.as_arg = as_arg;
-      c->uc = NULL;
     }
   return common_init (c, 0);
 #endif /* !UNW_LOCAL_ONLY */
