@@ -38,7 +38,7 @@ unw_is_signal_frame (unw_cursor_t *cursor)
 {
   /* XXXKIB */
   struct cursor *c = (struct cursor *) cursor;
-  unw_word_t w0, w1, w2, b0, ip;
+  unw_word_t w0, w1, w2, ip;
   unw_addr_space_t as;
   unw_accessors_t *a;
   void *arg;
@@ -69,22 +69,9 @@ eb fd                   jmp     0b
       w2 == 0x0000000000fdebf4)
    {
      c->sigcontext_format = X86_64_SCF_FREEBSD_SIGFRAME;
-     return (c->sigcontext_format);
+     return (1);
    }
-  /* Check if RIP points at standard syscall sequence.
-49 89 ca        mov    %rcx,%r10
-0f 05           syscall
-  */
-  if ((ret = (*a->access_mem) (as, ip - 5, &b0, 0, arg)) < 0)
-    return (0);
-  Debug (12, "b0 0x%lx\n", b0);
-  if ((b0 & 0xffffffffffffff) == 0x050fca89490000 ||
-      (b0 & 0xffffffffff) == 0x050fca8949)
-   {
-    c->sigcontext_format = X86_64_SCF_FREEBSD_SYSCALL;
-    return (c->sigcontext_format);
-   }
-  return (X86_64_SCF_NONE);
+  return (0);
 }
 
 HIDDEN int
@@ -92,7 +79,7 @@ x86_64_handle_signal_frame (unw_cursor_t *cursor)
 {
   struct cursor *c = (struct cursor *) cursor;
   unw_word_t ucontext;
-  int ret;
+  int i, ret;
 
   if (c->sigcontext_format == X86_64_SCF_FREEBSD_SIGFRAME)
    {
@@ -108,6 +95,9 @@ x86_64_handle_signal_frame (unw_cursor_t *cursor)
        return ret;
      }
 
+    for (i = 0; i < DWARF_NUM_PRESERVED_REGS; ++i)
+      c->dwarf.loc[i] = DWARF_NULL_LOC;
+
     c->dwarf.loc[RAX] = DWARF_LOC (ucontext + UC_MCONTEXT_GREGS_RAX, 0);
     c->dwarf.loc[RDX] = DWARF_LOC (ucontext + UC_MCONTEXT_GREGS_RDX, 0);
     c->dwarf.loc[RCX] = DWARF_LOC (ucontext + UC_MCONTEXT_GREGS_RCX, 0);
@@ -128,25 +118,6 @@ x86_64_handle_signal_frame (unw_cursor_t *cursor)
 
     return 0;
    }
-  else if (c->sigcontext_format == X86_64_SCF_FREEBSD_SYSCALL)
-   {
-    c->dwarf.loc[RCX] = c->dwarf.loc[R10];
-    /*  rsp_loc = DWARF_LOC(c->dwarf.cfa - 8, 0);       */
-    /*  rbp_loc = c->dwarf.loc[RBP];                    */
-    c->dwarf.loc[RIP] = DWARF_LOC (c->dwarf.cfa, 0);
-    ret = dwarf_get (&c->dwarf, c->dwarf.loc[RIP], &c->dwarf.ip);
-    Debug (1, "Frame Chain [RIP=0x%Lx] = 0x%Lx\n",
-           (unsigned long long) DWARF_GET_LOC (c->dwarf.loc[RIP]),
-           (unsigned long long) c->dwarf.ip);
-    if (ret < 0)
-     {
-       Debug (2, "returning %d\n", ret);
-       return ret;
-     }
-    c->dwarf.cfa += 8;
-    c->dwarf.use_prev_instr = 1;
-    return 1;
-   }
   else
     return -UNW_EBADFRAME;
 
@@ -216,3 +187,50 @@ x86_64_sigreturn (unw_cursor_t *cursor)
   abort();
 }
 #endif
+
+HIDDEN int
+x86_64_os_step(struct cursor *c)
+{
+  unw_word_t b0, ip;
+  unw_addr_space_t as;
+  unw_accessors_t *a;
+  void *arg;
+  int ret;
+
+  as = c->dwarf.as;
+  a = unw_get_accessors_int (as);
+  arg = c->dwarf.as_arg;
+  ip = c->dwarf.ip;
+
+  /*
+   * Check if RIP points at standard syscall sequence.
+   * 49 89 ca        mov    %rcx,%r10
+   * 0f 05           syscall
+   */
+  if ((ret = (*a->access_mem) (as, ip - 5, &b0, 0, arg)) < 0)
+    return (0);
+  Debug (12, "b0 0x%lx\n", b0);
+  if ((b0 & 0xffffffffffffff) == 0x050fca89490000 ||
+      (b0 & 0xffffffffff) == 0x050fca8949)
+   {
+    c->sigcontext_format = X86_64_SCF_FREEBSD_SYSCALL;
+    c->dwarf.loc[RCX] = c->dwarf.loc[R10];
+    /*  rsp_loc = DWARF_LOC(c->dwarf.cfa - 8, 0);       */
+    /*  rbp_loc = c->dwarf.loc[RBP];                    */
+    c->dwarf.loc[RSP] = DWARF_VAL_LOC (c, c->dwarf.cfa + 8);
+    c->dwarf.loc[RIP] = DWARF_LOC (c->dwarf.cfa, 0);
+    ret = dwarf_get (&c->dwarf, c->dwarf.loc[RIP], &c->dwarf.ip);
+    Debug (1, "Frame Chain [RIP=0x%Lx] = 0x%Lx\n",
+           (unsigned long long) DWARF_GET_LOC (c->dwarf.loc[RIP]),
+           (unsigned long long) c->dwarf.ip);
+    if (ret < 0)
+     {
+       Debug (2, "returning %d\n", ret);
+       return ret;
+     }
+    c->dwarf.cfa += 8;
+    c->dwarf.use_prev_instr = 1;
+    return 1;
+   }
+  return (0);
+}
