@@ -1,3 +1,5 @@
+#include "flush-cache.h"
+
 #include <assert.h>
 #include <libunwind.h>
 #include <unistd.h>
@@ -8,10 +10,6 @@
 
 #include <sys/mman.h>
 
-#ifdef HAVE_IA64INTRIN_H
-# include <ia64intrin.h>
-#endif
-
 int verbose;
 
 #ifdef __ia64__
@@ -24,8 +22,6 @@ int verbose;
 # define EXTRA			0
 #endif
 
-extern void flush_cache (void *addr, size_t len);
-
 int
 make_executable (void *addr, size_t len)
 {
@@ -50,7 +46,7 @@ create_func (unw_dyn_info_t *di, const char *name, long (*func) (),
   len = (uintptr_t) end - GET_ENTRY (func) + EXTRA;
   mem = malloc (len);
   if (verbose)
-    printf ("%s: cloning %s at %p (%Zu bytes)\n",
+    printf ("%s: cloning %s at %p (%zu bytes)\n",
 	    __FUNCTION__, name, mem, len);
   memend = (char *) mem + len;
 
