@@ -23,6 +23,13 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 
 /* Check whether basic unwinding truly is async-signal safe.  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include "compiler.h"
+
+#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -45,8 +52,16 @@ int verbose;
 int nerrors;
 int sigcount;
 
-#define panic(args...)					\
-	{ ++nerrors; fprintf (stderr, args); return; }
+#ifndef CONFIG_BLOCK_SIGNALS
+/* When libunwind is configured with --enable-block-signals=no, the caller
+   is responsible for preventing recursion via signal handlers.
+   We use a simple global here.  In a multithreaded program, one would use
+   a thread-local variable.  */
+int recurcount;
+#endif
+
+#define panic(...)					\
+	{ ++nerrors; fprintf (stderr, __VA_ARGS__); return; }
 
 static void
 do_backtrace (int may_print, int get_proc_name)
@@ -58,6 +73,12 @@ do_backtrace (int may_print, int get_proc_name)
   int ret;
   int depth = 0;
 
+#ifndef CONFIG_BLOCK_SIGNALS
+  if (recurcount > 0)
+    return;
+  recurcount += 1;
+#endif
+
   unw_getcontext (&uc);
   if (unw_init_local (&cursor, &uc) < 0)
     panic ("unw_init_local failed!\n");
@@ -70,7 +91,8 @@ do_backtrace (int may_print, int get_proc_name)
       buf[0] = '\0';
       if (get_proc_name || (may_print && verbose))
 	{
-	  if (unw_get_proc_name (&cursor, name, sizeof (name), &off) == 0)
+	  ret = unw_get_proc_name (&cursor, name, sizeof (name), &off);
+	  if (ret == 0 && (may_print && verbose))
 	    {
 	      if (off)
 		snprintf (buf, sizeof (buf), "<%s+0x%lx>", name, (long) off);
@@ -102,6 +124,10 @@ do_backtrace (int may_print, int get_proc_name)
         }
     }
   while (ret > 0);
+
+#ifndef CONFIG_BLOCK_SIGNALS
+  recurcount -= 1;
+#endif
 }
 
 void
@@ -131,11 +157,11 @@ sighandler (int signal)
 	printf ("SUCCESS.\n");
       exit (0);
     }
-  setitimer (ITIMER_VIRTUAL, &interval, NULL);
+  setitimer (ITIMER_REAL, &interval, NULL);
 }
 
 int
-main (int argc, char **argv)
+main (int argc, char **argv UNUSED)
 {
   struct sigaction act;
   long i = 0;
@@ -148,19 +174,20 @@ main (int argc, char **argv)
   memset (&act, 0, sizeof (act));
   act.sa_handler = sighandler;
   act.sa_flags = SA_SIGINFO;
-  sigaction (SIGVTALRM, &act, NULL);
+  sigaction (SIGALRM, &act, NULL);
 
-  setitimer (ITIMER_VIRTUAL, &interval, NULL);
+  setitimer (ITIMER_REAL, &interval, NULL);
 
   while (1)
     {
-      if (0 && verbose)
+      if (verbose)
 	printf ("%s: starting backtrace\n", __FUNCTION__);
       do_backtrace (0, (i++ % 100) == 0);
       if (nerrors > nerrors_max)
         {
-	  panic ("Too many errors (%d)\n", nerrors);
+	  fprintf (stderr, "Too many errors (%d)\n", nerrors);
 	  exit (-1);
         }
     }
+  return (0);
 }
