@@ -141,7 +141,7 @@ static void verror_msg_helper(const char *s,
   if (flags & LOGMODE_STDIO)
     {
       fflush(stdout);
-      write(STDERR_FILENO, msg, used + msgeol_len);
+      ssize_t written UNUSED = write(STDERR_FILENO, msg, used + msgeol_len);
     }
   msg[used] = '\0'; /* remove msg_eol (usually "\n") */
   if (flags & LOGMODE_SYSLOG)
@@ -244,8 +244,7 @@ void handle_sigsegv(int sig, siginfo_t *info, void *ucontext)
   {
     /* glibc extension */
     void *array[50];
-    int size;
-    size = backtrace(array, 50);
+    int size UNUSED = backtrace(array, 50);
 #if defined __linux__ && HAVE_EXECINFO_H
     backtrace_symbols_fd(array, size, 2);
 #endif
@@ -315,17 +314,6 @@ main(int argc UNUSED, char **argv)
     argv++;
   }
 
-  while (*argv)
-    {
-      char *colon;
-      unsigned long vaddr = strtoul(*argv, &colon, 16);
-      if (*colon != ':')
-        error_msg_and_die("Bad format: '%s'", *argv);
-      if (_UCD_add_backing_file_at_vaddr(ui, vaddr, colon + 1) < 0)
-        error_msg("Can't add backing file '%s'", colon + 1);
-      argv++;
-    }
-
   for (;;)
     {
       unw_word_t ip;
@@ -338,16 +326,23 @@ main(int argc UNUSED, char **argv)
       if (ret < 0)
         error_msg_and_die("unw_get_proc_info(ip=0x%lx) failed: ret=%d\n", (long) ip, ret);
 
-      if (!testcase) {
-        char proc_name[128];
-        unw_word_t off;
-        unw_get_proc_name(&c, proc_name, sizeof(proc_name), &off);
+      if (!testcase)
+        {
+          char proc_name[128];
+          unw_word_t off;
+          unw_get_proc_name(&c, proc_name, sizeof(proc_name), &off);
 
-        printf("\tip=0x%08lx proc=%08lx-%08lx handler=0x%08lx lsda=0x%08lx %s\n",
-				(long) ip,
-				(long) pi.start_ip, (long) pi.end_ip,
-				(long) pi.handler, (long) pi.lsda, proc_name);
-	  }
+          printf("\tip=0x%08lx proc=%08lx-%08lx handler=0x%08lx lsda=0x%08lx %s\n",
+                 (long) ip,
+                 (long) pi.start_ip, (long) pi.end_ip,
+                 (long) pi.handler, (long) pi.lsda, proc_name);
+
+          char filename[PATH_MAX];
+          unw_word_t file_offset;
+          ret = unw_get_elf_filename (&c, filename, sizeof (filename), &file_offset);
+          if (ret == UNW_ESUCCESS)
+              printf ("\t[%s+0x%lx]\n", filename, (long) file_offset);
+        }
 
       if (testcase && test_cur < TEST_FRAMES)
         {
